#include-once

; #FUNCTION# ==================================================================================
; Name...........: _ArrayCompare
; Description ...: Compares two one-dimensional arrays and returns an edit script for each one.
; Syntax.........: _ArrayCompare($aLeft, $aRight)
; Parameters ....: $aLeft        - The first array to be used for the comparison
;                  $aRight       - The second array to be used for the comparison
; Return values .: Success - A two element array with the script for left and right
;                  Failure - 0
;                  @Error  - 0 = No error
;                  |1 = $aLeft is not an array
;                  |2 = $aLeft is not one-dimensional
;                  |3 = $aRight is not an array
;                  |4 = $aRight is not one-dimensional
; Author ........: WideBoyDixon
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......; No
; =============================================================================================
Func _ArrayCompare(ByRef $aLeft, ByRef $aRight)
	; check for sensible data
	If Not IsArray($aLeft) Then Return SetError(1, 0, 0)
	If UBound($aLeft, 0) <> 1 Then Return SetError(2, 0, 0)
	If Not IsArray($aRight) Then Return SetError(3, 0, 0)
	If UBound($aRight, 0) <> 1 Then Return SetError(4, 0, 0)

	; create arrays for other functions
	Local $aLDD[UBound($aLeft) + 1][4]
	$aLDD[0][0] = UBound($aLeft)
	Local $aRDD[UBound($aRight) + 1][4]
	$aRDD[0][0] = UBound($aRight)
	For $i = 1 To UBound($aLeft)
		$aLDD[$i][0] = $aLeft[$i - 1]
		$aLDD[$i][1] = 0
	Next
	For $i = 1 To UBound($aRight)
		$aRDD[$i][0] = $aRight[$i - 1]
		$aRDD[$i][1] = 0
	Next

	; discard data that only appears in one array
	_DiscardConfusingEntries($aLDD, $aRDD)
	
	; compare sequences of non-discarded entries
	_FindSequences(0, $aLDD[0][1], 0, $aRDD[0][1], $aLDD, $aRDD)
	
	; copy the data over
	Local $aLeftScript[UBound($aLeft)]
	For $i = 1 To UBound($aLeft)
		$aLeftScript[$i - 1] = $aLDD[$i][1]
	Next
	Local $aRightScript[UBound($aRight)]
	For $i = 1 To UBound($aRight)
		$aRightScript[$i - 1] = $aRDD[$i][1]
	Next

	; success
	Local $ret[2] = [$aLeftScript, $aRightScript]
	Return SetError(0, 0, $ret)
EndFunc   ;==>_ArrayCompare

Func _DiscardConfusingEntries(ByRef $aODD, ByRef $aNDD)
	Local $i, $j, $fFound

	; Throw away old data
	$aODD[0][1] = 0
	For $i = 0 To $aODD[0][0] - 1
		; Search for this data in the new list
		$fFound = False
		For $j = 0 To $aNDD[0][0] - 1
			If $aODD[$i + 1][0] = $aNDD[$j + 1][0] Then
				; Found it!
				$fFound = True
				ExitLoop
			EndIf
		Next

		If $fFound Then
			; Don't discard it
			$aODD[0][1] += 1
			$aODD[$aODD[0][1]][2] = $aODD[$i + 1][0]
			$aODD[$aODD[0][1]][3] = $i
		Else
			; Not there in the new list ... must have been deleted
			$aODD[$i + 1][1] = 1
		EndIf
	Next

	; Throw away new data
	$aNDD[0][1] = 0
	For $i = 0 To $aNDD[0][0] - 1
		; Search for this data in the old list
		$fFound = False
		For $j = 0 To $aODD[0][0] - 1
			If $aNDD[$i + 1][0] = $aODD[$j + 1][0] Then
				; Found it!
				$fFound = True
				ExitLoop
			EndIf
		Next

		If $fFound Then
			; Don't discard it
			$aNDD[0][1] += 1
			$aNDD[$aNDD[0][1]][2] = $aNDD[$i + 1][0]
			$aNDD[$aNDD[0][1]][3] = $i
		Else
			; Not in old list ... must have been added
			$aNDD[$i + 1][1] = 1
		EndIf
	Next
EndFunc   ;==>_DiscardConfusingEntries

Func _FindSequences($iOldLo, $iOldHi, $iNewLo, $iNewHi, ByRef $aODD, ByRef $aNDD)
	Local $iLongestSegment, $iBest, $i, $j, $k, $i1, $j1, $fFound

	; Discard common data from the start
	While (($iOldLo < $iOldHi) And ($iNewLo < $iNewHi))
		If $aODD[$iOldLo + 1][2] = $aNDD[$iNewLo + 1][2] Then
			$iOldLo += 1
			$iNewLo += 1
		Else
			ExitLoop
		EndIf
	WEnd

	; Discard common data from the end
	While (($iOldHi > $iOldLo) And ($iNewHi > $iNewLo))
		If $aODD[$iOldHi][2] = $aNDD[$iNewHi][2] Then
			$iOldHi -= 1
			$iNewHi -= 1
		Else
			ExitLoop
		EndIf
	WEnd

	; Handle simple cases
	If ($iOldLo = $iOldHi) Then
		While ($iNewLo < $iNewHi)
			$aNDD[$aNDD[$iNewLo + 1][3] + 1][1] = 1
			$iNewLo += 1
		WEnd
		Return
	EndIf
	If ($iNewLo = $iNewHi) Then
		While ($iOldLo < $iOldHi)
			$aODD[$aODD[$iOldLo + 1][3] + 1][1] = 1
			$iOldLo += 1
		WEnd
		Return
	EndIf

	; String list [nOldLo ... nOldHi - 1] differs from [nNewLo ... nNewHi - 1]
	; Find a maximum correspondence between the two

	; Determine longest segment
	$iLongestSegment = $iOldHi - $iOldLo
	If ($iLongestSegment > ($iNewHi - $iNewLo)) Then $iLongestSegment = $iNewHi - $iNewLo
	If ($iLongestSegment <= 1) Then
		While ($iNewLo < $iNewHi)
			$aNDD[$aNDD[$iNewLo + 1][3] + 1][1] = 1
			$iNewLo += 1
		WEnd
		While ($iOldLo < $iOldHi)
			$aODD[$aODD[$iOldLo + 1][3] + 1][1] = 1
			$iOldLo += 1
		WEnd
		Return
	EndIf

	; Search for a correspondence between the data lists
	$iBest = 0
	For $i = $iOldLo To $iOldHi - 1
		If ($i + $iBest) >= $iOldHi Then ExitLoop
		For $j = $iNewLo To $iNewHi - 1
			If ($j + $iBest) >= $iNewHi Then ExitLoop
			If $aODD[$i + 1][2] = $aNDD[$j + 1][2] Then
				; Find length of this correspondence
				$k = 1
				While True
					If (($i + $k) < $iOldHi) And (($j + $k) < $iNewHi) Then
						If $aODD[$i + $k + 1][2] = $aNDD[$j + $k + 1][2] Then
							$k += 1
						Else
							ExitLoop
						EndIf
					Else
						ExitLoop
					EndIf
				WEnd
				If $k > $iBest Then
					$iBest = $k
					$i1 = $i
					$j1 = $j
				EndIf
			EndIf
		Next
	Next

	; Found a correspondence?
	If $iBest > 0 Then
		_FindSequences($iOldLo, $i1, $iNewLo, $j1, $aODD, $aNDD)
		_FindSequences($i1 + $iBest, $iOldHi, $j1 + $iBest, $iNewHi, $aODD, $aNDD)
	Else
		While ($iNewLo < $iNewHi)
			$aNDD[$aNDD[$iNewLo + 1][3] + 1][1] = 1
			$iNewLo += 1
		WEnd
		While ($iOldLo < $iOldHi)
			$aODD[$aODD[$iOldLo + 1][3] + 1][1] = 1
			$iOldLo += 1
		WEnd
	EndIf
EndFunc   ;==>_FindSequences
